from typing import Union

from robot.utils import timestr_to_secs  # type: ignore


def timestr_to_millisecs(time_str: Union[Union[int, str], float]) -> int:
    return timestr_to_secs(time_str) * 1000



import json
from enum import Enum, auto

from robotlibcore import keyword  # type: ignore
from typing import Optional, Dict, Any

from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils.time_conversion import timestr_to_millisecs


class MouseButton(Enum):
    left = auto()
    middle = auto()
    right = auto()


class KeyboardModifier(Enum):
    Alt = auto()
    Control = auto()
    Meta = auto()
    Shift = auto()


class SelectAttribute(Enum):
    value = auto()
    label = auto()
    text = label
    index = auto()


class Input(LibraryComponent):
    @keyword
    def type_text(
        self, selector: str, text: str, delay: str = "0 ms", clear: bool = True
    ):
        """Types the given ``text`` into the text field found by ``selector``.

        Sends a ``keydown``, ``keypress/input``, and ``keyup`` event for each
        character in the text.

        ``delay``: delay between the single key strokes. It may be either a
        number or a Robot Framework time string. Time strings are fully
        explained in an appendix of Robot Framework User Guide.
        Example: ``50 ms``

        See `Fill Text` for direct filling of the full text at once.
        """
        self._type_text(selector, text, delay, clear)

    @keyword
    def fill_text(self, selector: str, text: str):
        """Clears and fills the given ``text`` into the text field found by ``selector``.

        This method waits for an element matching the ``selector`` to appear,
        waits for actionability checks, focuses the element, fills it and
        triggers an input event after filling.

        If the element matching selector is not an <input>, <textarea> or
        [contenteditable] element, this method throws an error. Note that
        you can pass an empty string as ``text`` to clear the input field.

        See `Type Text` for emulating typing text character by character.
        """
        self._fill_text(selector, text)

    @keyword
    def clear_text(self, selector: str):
        """Clears the text field found by ``selector``.

        See `Type Text` for emulating typing text character by character.
        See `Fill Text` for direct filling of the full text at once.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ClearText(Request().ClearText(selector=selector))
            self.info(response.log)

    @keyword
    def type_secret(
        self, selector: str, secret: str, delay: str = "0 ms", clear: bool = True
    ):
        """Types the given ``secret`` into the text field found by ``selector``.

        The difference to `Type Text` is that this keyword does not log the
        text to be written into the text field.

        See `Type Text` for details.
        """
        self._type_text(selector, secret, delay, clear, log_response=False)

    @keyword
    def fill_secret(self, selector: str, secret: str):
        """Fills the given ``secret`` into the text field found by ``selector``.

        The difference to `Fill Text` is that this keyword does not log the
        text to be written into the text field.

        See `Fill Text` for details.
        """
        self._fill_text(selector, secret, log_response=False)

    @keyword
    def press_keys(self, selector: str, *keys: str):
        """Types the given key combination into element found by ``selector``.

        Supports values like "a, b" which will be automatically typed.
        .
        Also supports identifiers for keys like ``ArrowLeft`` or ``Backspace``.
        Using + to chain combine modifiers with a single keypress
        ``Control+Shift+T`` is supported.

        See playwright's documentation for a more comprehensive list of
        supported input keys.
        [https://github.com/microsoft/playwright/blob/master/docs/api.md#pagepressselector-key-options | Playwright docs for press.]
        """  # noqa
        with self.playwright.grpc_channel() as stub:
            response = stub.Press(Request().PressKeys(selector=selector, key=keys))
            self.info(response.log)

    @keyword
    def click(self, selector: str):
        """Clicks the element found by ``selector``."""
        with self.playwright.grpc_channel() as stub:
            response = stub.Click(Request().ElementSelector(selector=selector))
            self.info(response.log)

    @keyword
    def click_with_options(
        self,
        selector: str,
        button: MouseButton = MouseButton.left,
        click_count: int = 1,
        delay: Optional[str] = None,
        position_x: Optional[int] = None,
        position_y: Optional[int] = None,
        *modifiers: KeyboardModifier,
    ):
        """Simulates mouse click on the element found by ``selector``.

        ``button``: ``<"left"|"right"|"middle">`` Defaults to ``left`` if invalid.

        ``clickCount``: <int> defaults to 1.

        ``delay``: <robot time> Time to wait between mousedown and mouseup.
        Defaults to 0.

        ``position_x`` & ``position_y``: <int> A point to click relative to the
        top-left corner of element padding box.
        If not specified, clicks to some visible point of the element.

        ``*modifiers``: ``<list<"Alt"|"Control"|"Meta"|"Shift">>``
        Modifier keys to press. Ensures that only these modifiers are pressed
        during the click, and then restores current modifiers back.
        If not specified, currently pressed modifiers are used.
        """
        with self.playwright.grpc_channel() as stub:
            options = {"button": button.name, "clickCount": click_count}
            if delay:
                options["delay"] = timestr_to_millisecs(delay)
            if position_x and position_y:
                positions: Dict[str, object] = {"x": position_x, "y": position_y}
                options["position"] = positions
            if modifiers:
                options["modifiers"] = [m.name for m in modifiers]
            options_json = json.dumps(options)
            self.debug(f"Click Options are: {options_json}")
            response = stub.Click(
                Request().ElementSelectorOptions(
                    selector=selector, options=options_json
                )
            )
            self.info(response.log)

    @keyword
    def focus(self, selector: str):
        """Moves focus on to the element found by ``selector``.

        If there's no element matching selector, the method waits until a
        matching element appears in the DOM.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.Focus(Request().ElementSelector(selector=selector))
            self.info(response.log)

    @keyword
    def execute_javascript_on_page(self, script: str) -> Any:
        """Executes given javascript on the page.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(script=script)
            )
            self.info(response.log)
            return json.loads(response.result)

    @keyword
    def check_checkbox(self, selector: str):
        """Checks the checkbox or selects radio button found by ``selector``.

        Does nothing if the element is already checked/selected.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.CheckCheckbox(Request().ElementSelector(selector=selector))
            self.info(response.log)

    @keyword
    def uncheck_checkbox(self, selector: str):
        """Unchecks the checkbox found by ``selector``.

        Does nothing if the element is not checked/selected.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.UncheckCheckbox(
                Request().ElementSelector(selector=selector)
            )
            self.info(response.log)

    @keyword
    def select_options_by(self, attribute: SelectAttribute, selector: str, *values):
        """Selects options from select element found by ``selector``.

        Matches based on the chosen attribute with list of ``values``.
        Possible attributes to match options by:
        ``attribute``: ``<"value"|"label"|"text"|"index">``

        If no values to select are passed will deselect options in element.
        """
        matchers = ""
        if not values or len(values) == 1 and not values[0]:
            self.deselect_options(selector)
            return

        if attribute is SelectAttribute.value:
            matchers = json.dumps([{"value": s} for s in values])
        elif attribute is SelectAttribute.label:
            matchers = json.dumps([{"label": s} for s in values])
        elif attribute is SelectAttribute.index:
            matchers = json.dumps([{"index": int(s)} for s in values])
        with self.playwright.grpc_channel() as stub:
            response = stub.SelectOption(
                Request().SelectElementSelector(selector=selector, matcherJson=matchers)
            )
            self.info(response.log)

    @keyword
    def deselect_options(self, selector: str):
        """Deselects all options from select element found by ``selector``."""
        with self.playwright.grpc_channel() as stub:
            response = stub.DeselectOption(Request().ElementSelector(selector=selector))
            self.info(response.log)

    def _fill_text(self, selector: str, text: str, log_response: bool = True):
        with self.playwright.grpc_channel() as stub:
            response = stub.FillText(Request().FillText(selector=selector, text=text))
            if log_response:
                self.info(response.log)

    def _type_text(
        self,
        selector: str,
        text: str,
        delay: str = "0 ms",
        clear: bool = True,
        log_response: bool = True,
    ):
        with self.playwright.grpc_channel() as stub:
            delay_ms = timestr_to_millisecs(delay)
            response = stub.TypeText(
                Request().TypeText(
                    selector=selector, text=text, delay=int(delay_ms), clear=clear
                )
            )
            if log_response:
                self.info(response.log)



import re
import os

from robot.api import logger  # type: ignore
from robot.libraries.BuiltIn import BuiltIn, EXECUTION_CONTEXTS  # type: ignore
from robotlibcore import DynamicCore  # type: ignore

from .keywords import BrowserState, Control, Getters, Input, Waiter, WebAppState
from .playwright import Playwright
from .version import VERSION

__version__ = VERSION


class Browser(DynamicCore):
    """Browser library is a browser automation library for Robot Framework.

    This is hte keyword documentation for Browser library. For information
    about installation, support, and more please visit the
    [https://github.com/MarketSquare/robotframework-playwright|project pages].
    For more information about Robot Framework itself, see [https://robotframework.org|robotframework.org].

    Browser library uses
    [https://github.com/microsoft/playwright|Playwright Node module]
    to automate [https://www.chromium.org/Home|Chromium],
    [https://www.mozilla.org/en-US/firefox/new/|Firefox]
    and [https://webkit.org/|WebKit] with a single library.

    == Table of contents ==

    %TOC%

    = Finding elements =

    All keywords in hte library that need to interact with an element
    on a web page take an argument typically named ``selector`` that specifies
    how to find the element.

    == Selector syntax ==

    Browser library supports the same selector strategies as the underlying
    Playwright node module: xpath, css, id and text. The strategy can either
    be explicitly specified with a prefix or the strategy can be implicit.

    === Implicit selector strategy ===

    The default selector strategy is `css`. If selector does not contain
    one of the know selector strategies, `css`, `xpath`, `id` or `text` it is
    assumed to contain css selector. Also `selectors` starting with `//`
    considered as xpath selectors.

    Examples:

    | `Click` | span > button | # Use css selector strategy the element.   |
    | `Click` | //span/button | # Use xpath selector strategy the element. |

    === Explicit selector strategy ===

    The explicit selector strategy is specified with a prefix using syntax
    ``strategy=value``. Spaces around the separator are ignored, so
    ``css=foo``, ``css= foo`` and ``css = foo`` are all equivalent.


    Selector strategies that are supported by default are listed in the table
    below.

    | = Strategy = |     = Match based on =     |         = Example =            |
    | css          | CSS selector.              | ``css=div#example``            |
    | xpath        | XPath expression.          | ``xpath=//div[@id="example"]`` |
    | text         | Browser text engine.       | ``text=Login``                 |

    == Finding elements inside frames ==

    By default, selector chains do not cross frame boundaries. It means that a
    simple CSS selector is not able to select and element located inside an iframe
    or a frameset. For this usecase, there is a special selector ``>>>`` which can
    be used to combine a selector for the frame and a selector for an element
    inside a frame.

    Given this simple pseudo html snippet:
    ``<iframe name="iframe" src="src.html"><button id="btn">Click Me</button></iframe>``,
    here's a keyword call that clicks the button inside the frame.

    | Click  | iframe[name="iframe"] >>> #btn  |

    The selectors on the left and right side of ``>>>`` can be any valid selectors.

    = Assertions =

    Keywords that accept arguments ``assertion_operator`` and ``assertion_expected``
    can optionally assert.
    Currently supported assertion operators are:

    |      = Operator =               |              = Description =                          |
    | ``==`` or ``should be``         | equal                                                 |
    | ``!=`` or ``should not be``     | not equal                                             |
    | ``>``                           | greater than                                          |
    | ``>=``                          | greater than or equal                                 |
    | ``<``                           | less than                                             |
    | ``<=``                          | less than or equal                                    |
    | ``*=`` or ``contains``          | for checking that a value contains an element         |
    | ``matches``                     | for matching against a regular expression.            |
    | ``^=`` or ``should start with`` | starts with                                           |
    | ``$=`` or ``should end with``   | ends with                                             |
    | ``validate``                    | use BuiltIn Evaluate. Access to actual with ``value`` |

    Assertion value can be any valid robot value, and the keywords will provide an error
    message if the assertion fails.

    = The 'then' closure =

    Keywords that accept arguments ``assertion_operator`` and ``assertion_expected``
    can optionally also use ``then`` closure to modify the returned value with
    BuiltIn Evaluate. Actual value can be accessed with ``value``.

    For example ``Get Title  then  'TITLE: '+value``.
    See
    [https://robotframework.org/robotframework/latest/libraries/BuiltIn.html#Evaluating%20expressions|
    Builtin Evaluating expressions]
    for more info on the syntax.

    """

    ROBOT_LIBRARY_VERSION = __version__
    ROBOT_LISTENER_API_VERSION = 2
    ROBOT_LIBRARY_LISTENER: "Browser"
    ROBOT_LIBRARY_SCOPE = "GLOBAL"
    SUPPORTED_BROWSERS = ["chromium", "firefox", "webkit"]

    def __init__(self, timeout="10s", enable_playwright_debug=False):
        """Browser library can be taken into use with optional arguments:

        - ``timeout``:
          Timeout for keywords that operate on elements. The keywords will wait
          for this time for the element to appear into the page.
        - ``enable_playwright_debug``:
          Enable low level debug information from the playwright tool. Mainly
          Useful for the library developers and for debugging purposes.
        """
        self.ROBOT_LIBRARY_LISTENER = self
        self.browser_control = Control(self)
        libraries = [
            self.browser_control,
            BrowserState(self),
            Input(self),
            Getters(self),
            Waiter(self),
            WebAppState(self),
        ]
        self.playwright = Playwright(timeout, enable_playwright_debug)
        DynamicCore.__init__(self, libraries)

    @property
    def outputdir(self):
        return BuiltIn().get_variable_value("${OUTPUTDIR}")

    def _close(self):
        self.playwright.close()

    def run_keyword(self, name, args, kwargs=None):
        try:
            return DynamicCore.run_keyword(self, name, args, kwargs)
        except AssertionError as e:
            self.keyword_error()
            raise e

    def start_keyword(self, name, attrs):
        """Take screenshot of tests that have failed due to timeout.

        This method is part of the Listener API implemented by the library.

        This can be done with BuiltIn keyword `Run Keyword If Timeout
        Occurred`, but the problem there is that you have to remember to
        put it into your Suite/Test Teardown. Since taking screenshot is
        the most obvious thing to do on failure, let's do it automatically.

        This cannot be implemented as a `end_test` listener method, since at
        that time, the teardown has already been executed and browser may have
        been closed already. This implementation will take the screenshot
        before the teardown begins to execute.
        """
        if attrs["type"] == "Teardown":
            timeout_pattern = "Test timeout .* exceeded."
            test = EXECUTION_CONTEXTS.current.test
            if (
                test is not None
                and test.status == "FAIL"
                and re.match(timeout_pattern, test.message)
            ):
                self.screenshot_on_failure(test.name)

    def keyword_error(self):
        """Sends screenshot command to Playwright.

        Only works during testing since this uses robot's outputdir for output.
        """
        self.screenshot_on_failure(BuiltIn().get_variable_value("${TEST NAME}"))

    def screenshot_on_failure(self, test_name):
        try:
            path = self.failure_screenshot_path(test_name)
            self.browser_control.take_page_screenshot(path)
        except Exception as err:
            logger.warn(f"Was unable to take page screenshot after failure:\n{err}")

    def failure_screenshot_path(self, test_name):
        return os.path.join(
            BuiltIn().get_variable_value("${OUTPUTDIR}"),
            test_name.replace(" ", "_") + "_FAILURE_SCREENSHOT",
        ).replace("\\", "\\\\")



from enum import Enum
from typing import Any, Dict, Tuple, Callable, TypeVar, cast, Optional
from robot.libraries.BuiltIn import BuiltIn  # type: ignore
import re

AssertionOperator = Enum(
    "AssertionOperator",
    {
        "equal": "==",
        "==": "==",
        "shouldbe": "==",
        "inequal": "!=",
        "!=": "!=",
        "shouldnotbe": "!=",
        "lessthan": "<",
        "<": "<",
        "greaterthan": ">",
        ">": ">",
        "<=": "<=",
        ">=": ">=",
        "contains": "*=",
        "*=": "*=",
        "starts": "^=",
        "^=": "^=",
        "shouldstartwith": "^=",
        "ends": "$=",
        "$=": "$=",
        "matches": "$",
        "validate": "validate",
        "then": "then",
    },
)


handlers: Dict[AssertionOperator, Tuple[Callable, str]] = {
    AssertionOperator["=="]: (lambda a, b: a == b, "should be"),
    AssertionOperator["!="]: (lambda a, b: a != b, "should not be"),
    AssertionOperator["<"]: (lambda a, b: a < b, "should be less than"),
    AssertionOperator[">"]: (lambda a, b: a > b, "should be greater than"),
    AssertionOperator["<="]: (lambda a, b: a <= b, "should be less than or equal"),
    AssertionOperator[">="]: (lambda a, b: a >= b, "should be greater than or equal"),
    AssertionOperator["*="]: (lambda a, b: b in a, "should contain"),
    AssertionOperator["matches"]: (lambda a, b: re.search(b, a), "should match"),
    AssertionOperator["^="]: (
        lambda a, b: re.search(f"^{re.escape(b)}", a),
        "should start with",
    ),
    AssertionOperator["$="]: (
        lambda a, b: re.search(f"{re.escape(b)}$", a),
        "should end with",
    ),
    AssertionOperator["validate"]: (
        lambda a, b: BuiltIn().evaluate(b, namespace={"value": a}),
        "should validate to true with",
    ),
}

T = TypeVar("T")


def verify_assertion(
    value: T, operator: Optional[AssertionOperator], expected: Any, message=""
) -> Any:
    if operator is None:
        return value
    if operator is AssertionOperator["then"]:
        return cast(T, BuiltIn().evaluate(expected, namespace={"value": value}))
    handler = handlers.get(operator)
    if handler is None:
        raise RuntimeError(f"{message} `{operator}` is not a valid assertion operator")
    validator, text = handler
    if not validator(value, expected):
        raise AssertionError(f"{message} `{value}` {text} `{expected}`")
    return value



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: playwright.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='playwright.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10playwright.proto\"\xd2\x06\n\x07Request\x1a\x07\n\x05\x45mpty\x1a\x1e\n\x0eScreenshotPath\x12\x0c\n\x04path\x18\x01 \x01(\t\x1a@\n\x07\x42rowser\x12\x0f\n\x07\x62rowser\x18\x01 \x01(\t\x12\x10\n\x08headless\x18\x02 \x01(\x08\x12\x12\n\nrawOptions\x18\x03 \x01(\t\x1a\x1d\n\x07\x43ontext\x12\x12\n\nrawOptions\x18\x01 \x01(\t\x1a\x12\n\x03Url\x12\x0b\n\x03url\x18\x01 \x01(\t\x1a:\n\tTextInput\x12\r\n\x05input\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x08\x1a\x35\n\x0f\x45lementProperty\x12\x10\n\x08property\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x1aH\n\x08TypeText\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\r\n\x05\x64\x65lay\x18\x03 \x01(\x05\x12\r\n\x05\x63lear\x18\x04 \x01(\x08\x1a*\n\x08\x46illText\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x1a\x1d\n\tClearText\x12\x10\n\x08selector\x18\x01 \x01(\t\x1a*\n\tPressKeys\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x03(\t\x1a#\n\x0f\x45lementSelector\x12\x10\n\x08selector\x18\x01 \x01(\t\x1a\x1a\n\x07Timeout\x12\x0f\n\x07timeout\x18\x01 \x01(\x02\x1a\x16\n\x05Index\x12\r\n\x05index\x18\x01 \x01(\x05\x1a\x1b\n\x08StyleTag\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x1a?\n\x1a\x45lementSelectorWithOptions\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0f\n\x07options\x18\x02 \x01(\t\x1a\x41\n\x1b\x45lementSelectorWithDuration\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x10\n\x08\x64uration\x18\x02 \x01(\x05\x1a>\n\x15SelectElementSelector\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x13\n\x0bmatcherJson\x18\x02 \x01(\t\x1a \n\x0eJavascriptCode\x12\x0e\n\x06script\x18\x01 \x01(\t\x1a\x19\n\x07RawJson\x12\x0e\n\x06script\x18\x01 \x01(\t\"=\n\x0bSelectEntry\x12\r\n\x05value\x18\x02 \x01(\t\x12\r\n\x05label\x18\x03 \x01(\t\x12\x10\n\x08selected\x18\x04 \x01(\x08\"\xeb\x01\n\x08Response\x1a\x14\n\x05\x45mpty\x12\x0b\n\x03log\x18\x01 \x01(\t\x1a#\n\x06String\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\t\x1a!\n\x04\x42ool\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x08\x1a \n\x03Int\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x05\x1a%\n\x06Select\x12\x1b\n\x05\x65ntry\x18\x01 \x03(\x0b\x32\x0c.SelectEntry\x1a\x38\n\x19JavascriptExecutionResult\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\t2\x93\x11\n\nPlaywright\x12=\n\x0eTakeScreenshot\x12\x17.Request.ScreenshotPath\x1a\x10.Response.String\"\x00\x12\x31\n\x0c\x43loseBrowser\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\"\x00\x12\'\n\x04GoTo\x12\x0c.Request.Url\x1a\x0f.Response.Empty\"\x00\x12)\n\x06GoBack\x12\x0c.Request.Url\x1a\x0f.Response.Empty\"\x00\x12,\n\tGoForward\x12\x0c.Request.Url\x1a\x0f.Response.Empty\"\x00\x12.\n\x08GetTitle\x12\x0e.Request.Empty\x1a\x10.Response.String\"\x00\x12<\n\x0fGetElementCount\x12\x18.Request.ElementSelector\x1a\r.Response.Int\"\x00\x12\x32\n\tInputText\x12\x12.Request.TextInput\x1a\x0f.Response.Empty\"\x00\x12\x30\n\x08TypeText\x12\x11.Request.TypeText\x1a\x0f.Response.Empty\"\x00\x12\x30\n\x08\x46illText\x12\x11.Request.FillText\x1a\x0f.Response.Empty\"\x00\x12\x32\n\tClearText\x12\x12.Request.ClearText\x1a\x0f.Response.Empty\"\x00\x12>\n\x0eGetDomProperty\x12\x18.Request.ElementProperty\x1a\x10.Response.String\"\x00\x12=\n\x0fGetBoolProperty\x12\x18.Request.ElementProperty\x1a\x0e.Response.Bool\"\x00\x12>\n\x0eGetTextContent\x12\x18.Request.ElementSelector\x1a\x10.Response.String\"\x00\x12,\n\x06GetUrl\x12\x0e.Request.Empty\x1a\x10.Response.String\"\x00\x12\x34\n\x05\x43lick\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\"\x00\x12.\n\x05Press\x12\x12.Request.PressKeys\x1a\x0f.Response.Empty\"\x00\x12@\n\x10GetSelectContent\x12\x18.Request.ElementSelector\x1a\x10.Response.Select\"\x00\x12\x41\n\x0cSelectOption\x12\x1e.Request.SelectElementSelector\x1a\x0f.Response.Empty\"\x00\x12=\n\x0e\x44\x65selectOption\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\"\x00\x12<\n\rCheckCheckbox\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\"\x00\x12>\n\x0fUncheckCheckbox\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\"\x00\x12,\n\x06Health\x12\x0e.Request.Empty\x1a\x10.Response.String\"\x00\x12\x31\n\nSetTimeout\x12\x10.Request.Timeout\x1a\x0f.Response.Empty\"\x00\x12\x33\n\x0b\x41\x64\x64StyleTag\x12\x11.Request.StyleTag\x1a\x0f.Response.Empty\"\x00\x12L\n\x11HighlightElements\x12$.Request.ElementSelectorWithDuration\x1a\x0f.Response.Empty\"\x00\x12J\n\x10\x43lickWithOptions\x12#.Request.ElementSelectorWithOptions\x1a\x0f.Response.Empty\"\x00\x12\x34\n\x05\x46ocus\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\"\x00\x12N\n\x14WaitForElementsState\x12#.Request.ElementSelectorWithOptions\x1a\x0f.Response.Empty\"\x00\x12Y\n\x17\x45xecuteJavascriptOnPage\x12\x17.Request.JavascriptCode\x1a#.Response.JavascriptExecutionResult\"\x00\x12\x45\n\x0cGetPageState\x12\x0e.Request.Empty\x1a#.Response.JavascriptExecutionResult\"\x00\x12\x36\n\x11\x41utoActivatePages\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\"\x00\x12/\n\nSwitchPage\x12\x0e.Request.Index\x1a\x0f.Response.Empty\"\x00\x12\x32\n\rSwitchContext\x12\x0e.Request.Index\x1a\x0f.Response.Empty\"\x00\x12\x32\n\rSwitchBrowser\x12\x0e.Request.Index\x1a\x0f.Response.Empty\"\x00\x12-\n\nCreatePage\x12\x0c.Request.Url\x1a\x0f.Response.Empty\"\x00\x12\x34\n\rCreateContext\x12\x10.Request.Context\x1a\x0f.Response.Empty\"\x00\x12\x34\n\rCreateBrowser\x12\x10.Request.Browser\x1a\x0f.Response.Empty\"\x00\x62\x06proto3'
)




_REQUEST_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Request.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32,
  serialized_end=39,
)

_REQUEST_SCREENSHOTPATH = _descriptor.Descriptor(
  name='ScreenshotPath',
  full_name='Request.ScreenshotPath',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='Request.ScreenshotPath.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=71,
)

_REQUEST_BROWSER = _descriptor.Descriptor(
  name='Browser',
  full_name='Request.Browser',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='browser', full_name='Request.Browser.browser', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='headless', full_name='Request.Browser.headless', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rawOptions', full_name='Request.Browser.rawOptions', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=137,
)

_REQUEST_CONTEXT = _descriptor.Descriptor(
  name='Context',
  full_name='Request.Context',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rawOptions', full_name='Request.Context.rawOptions', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=139,
  serialized_end=168,
)

_REQUEST_URL = _descriptor.Descriptor(
  name='Url',
  full_name='Request.Url',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='Request.Url.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=188,
)

_REQUEST_TEXTINPUT = _descriptor.Descriptor(
  name='TextInput',
  full_name='Request.TextInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='Request.TextInput.input', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.TextInput.selector', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='Request.TextInput.type', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=248,
)

_REQUEST_ELEMENTPROPERTY = _descriptor.Descriptor(
  name='ElementProperty',
  full_name='Request.ElementProperty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='property', full_name='Request.ElementProperty.property', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.ElementProperty.selector', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=303,
)

_REQUEST_TYPETEXT = _descriptor.Descriptor(
  name='TypeText',
  full_name='Request.TypeText',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.TypeText.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='Request.TypeText.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delay', full_name='Request.TypeText.delay', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clear', full_name='Request.TypeText.clear', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=377,
)

_REQUEST_FILLTEXT = _descriptor.Descriptor(
  name='FillText',
  full_name='Request.FillText',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.FillText.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='Request.FillText.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=379,
  serialized_end=421,
)

_REQUEST_CLEARTEXT = _descriptor.Descriptor(
  name='ClearText',
  full_name='Request.ClearText',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.ClearText.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=423,
  serialized_end=452,
)

_REQUEST_PRESSKEYS = _descriptor.Descriptor(
  name='PressKeys',
  full_name='Request.PressKeys',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.PressKeys.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key', full_name='Request.PressKeys.key', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=454,
  serialized_end=496,
)

_REQUEST_ELEMENTSELECTOR = _descriptor.Descriptor(
  name='ElementSelector',
  full_name='Request.ElementSelector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.ElementSelector.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=498,
  serialized_end=533,
)

_REQUEST_TIMEOUT = _descriptor.Descriptor(
  name='Timeout',
  full_name='Request.Timeout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeout', full_name='Request.Timeout.timeout', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=535,
  serialized_end=561,
)

_REQUEST_INDEX = _descriptor.Descriptor(
  name='Index',
  full_name='Request.Index',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='Request.Index.index', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=563,
  serialized_end=585,
)

_REQUEST_STYLETAG = _descriptor.Descriptor(
  name='StyleTag',
  full_name='Request.StyleTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='Request.StyleTag.content', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=587,
  serialized_end=614,
)

_REQUEST_ELEMENTSELECTORWITHOPTIONS = _descriptor.Descriptor(
  name='ElementSelectorWithOptions',
  full_name='Request.ElementSelectorWithOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.ElementSelectorWithOptions.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='Request.ElementSelectorWithOptions.options', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=616,
  serialized_end=679,
)

_REQUEST_ELEMENTSELECTORWITHDURATION = _descriptor.Descriptor(
  name='ElementSelectorWithDuration',
  full_name='Request.ElementSelectorWithDuration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.ElementSelectorWithDuration.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration', full_name='Request.ElementSelectorWithDuration.duration', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=681,
  serialized_end=746,
)

_REQUEST_SELECTELEMENTSELECTOR = _descriptor.Descriptor(
  name='SelectElementSelector',
  full_name='Request.SelectElementSelector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='Request.SelectElementSelector.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='matcherJson', full_name='Request.SelectElementSelector.matcherJson', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=748,
  serialized_end=810,
)

_REQUEST_JAVASCRIPTCODE = _descriptor.Descriptor(
  name='JavascriptCode',
  full_name='Request.JavascriptCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script', full_name='Request.JavascriptCode.script', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=812,
  serialized_end=844,
)

_REQUEST_RAWJSON = _descriptor.Descriptor(
  name='RawJson',
  full_name='Request.RawJson',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='script', full_name='Request.RawJson.script', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=846,
  serialized_end=871,
)

_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_REQUEST_EMPTY, _REQUEST_SCREENSHOTPATH, _REQUEST_BROWSER, _REQUEST_CONTEXT, _REQUEST_URL, _REQUEST_TEXTINPUT, _REQUEST_ELEMENTPROPERTY, _REQUEST_TYPETEXT, _REQUEST_FILLTEXT, _REQUEST_CLEARTEXT, _REQUEST_PRESSKEYS, _REQUEST_ELEMENTSELECTOR, _REQUEST_TIMEOUT, _REQUEST_INDEX, _REQUEST_STYLETAG, _REQUEST_ELEMENTSELECTORWITHOPTIONS, _REQUEST_ELEMENTSELECTORWITHDURATION, _REQUEST_SELECTELEMENTSELECTOR, _REQUEST_JAVASCRIPTCODE, _REQUEST_RAWJSON, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=21,
  serialized_end=871,
)


_SELECTENTRY = _descriptor.Descriptor(
  name='SelectEntry',
  full_name='SelectEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='SelectEntry.value', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='SelectEntry.label', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selected', full_name='SelectEntry.selected', index=2,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=873,
  serialized_end=934,
)


_RESPONSE_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Response.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.Empty.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=949,
  serialized_end=969,
)

_RESPONSE_STRING = _descriptor.Descriptor(
  name='String',
  full_name='Response.String',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.String.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='Response.String.body', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=971,
  serialized_end=1006,
)

_RESPONSE_BOOL = _descriptor.Descriptor(
  name='Bool',
  full_name='Response.Bool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.Bool.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='Response.Bool.body', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1008,
  serialized_end=1041,
)

_RESPONSE_INT = _descriptor.Descriptor(
  name='Int',
  full_name='Response.Int',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.Int.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='Response.Int.body', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1043,
  serialized_end=1075,
)

_RESPONSE_SELECT = _descriptor.Descriptor(
  name='Select',
  full_name='Response.Select',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='entry', full_name='Response.Select.entry', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1077,
  serialized_end=1114,
)

_RESPONSE_JAVASCRIPTEXECUTIONRESULT = _descriptor.Descriptor(
  name='JavascriptExecutionResult',
  full_name='Response.JavascriptExecutionResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='log', full_name='Response.JavascriptExecutionResult.log', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='Response.JavascriptExecutionResult.result', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1116,
  serialized_end=1172,
)

_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_RESPONSE_EMPTY, _RESPONSE_STRING, _RESPONSE_BOOL, _RESPONSE_INT, _RESPONSE_SELECT, _RESPONSE_JAVASCRIPTEXECUTIONRESULT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=937,
  serialized_end=1172,
)

_REQUEST_EMPTY.containing_type = _REQUEST
_REQUEST_SCREENSHOTPATH.containing_type = _REQUEST
_REQUEST_BROWSER.containing_type = _REQUEST
_REQUEST_CONTEXT.containing_type = _REQUEST
_REQUEST_URL.containing_type = _REQUEST
_REQUEST_TEXTINPUT.containing_type = _REQUEST
_REQUEST_ELEMENTPROPERTY.containing_type = _REQUEST
_REQUEST_TYPETEXT.containing_type = _REQUEST
_REQUEST_FILLTEXT.containing_type = _REQUEST
_REQUEST_CLEARTEXT.containing_type = _REQUEST
_REQUEST_PRESSKEYS.containing_type = _REQUEST
_REQUEST_ELEMENTSELECTOR.containing_type = _REQUEST
_REQUEST_TIMEOUT.containing_type = _REQUEST
_REQUEST_INDEX.containing_type = _REQUEST
_REQUEST_STYLETAG.containing_type = _REQUEST
_REQUEST_ELEMENTSELECTORWITHOPTIONS.containing_type = _REQUEST
_REQUEST_ELEMENTSELECTORWITHDURATION.containing_type = _REQUEST
_REQUEST_SELECTELEMENTSELECTOR.containing_type = _REQUEST
_REQUEST_JAVASCRIPTCODE.containing_type = _REQUEST
_REQUEST_RAWJSON.containing_type = _REQUEST
_RESPONSE_EMPTY.containing_type = _RESPONSE
_RESPONSE_STRING.containing_type = _RESPONSE
_RESPONSE_BOOL.containing_type = _RESPONSE
_RESPONSE_INT.containing_type = _RESPONSE
_RESPONSE_SELECT.fields_by_name['entry'].message_type = _SELECTENTRY
_RESPONSE_SELECT.containing_type = _RESPONSE
_RESPONSE_JAVASCRIPTEXECUTIONRESULT.containing_type = _RESPONSE
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['SelectEntry'] = _SELECTENTRY
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {

  'Empty' : _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_EMPTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Empty)
    })
  ,

  'ScreenshotPath' : _reflection.GeneratedProtocolMessageType('ScreenshotPath', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_SCREENSHOTPATH,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ScreenshotPath)
    })
  ,

  'Browser' : _reflection.GeneratedProtocolMessageType('Browser', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_BROWSER,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Browser)
    })
  ,

  'Context' : _reflection.GeneratedProtocolMessageType('Context', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CONTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Context)
    })
  ,

  'Url' : _reflection.GeneratedProtocolMessageType('Url', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_URL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Url)
    })
  ,

  'TextInput' : _reflection.GeneratedProtocolMessageType('TextInput', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TEXTINPUT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.TextInput)
    })
  ,

  'ElementProperty' : _reflection.GeneratedProtocolMessageType('ElementProperty', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTPROPERTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementProperty)
    })
  ,

  'TypeText' : _reflection.GeneratedProtocolMessageType('TypeText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TYPETEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.TypeText)
    })
  ,

  'FillText' : _reflection.GeneratedProtocolMessageType('FillText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_FILLTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.FillText)
    })
  ,

  'ClearText' : _reflection.GeneratedProtocolMessageType('ClearText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CLEARTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ClearText)
    })
  ,

  'PressKeys' : _reflection.GeneratedProtocolMessageType('PressKeys', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_PRESSKEYS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.PressKeys)
    })
  ,

  'ElementSelector' : _reflection.GeneratedProtocolMessageType('ElementSelector', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTOR,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelector)
    })
  ,

  'Timeout' : _reflection.GeneratedProtocolMessageType('Timeout', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TIMEOUT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Timeout)
    })
  ,

  'Index' : _reflection.GeneratedProtocolMessageType('Index', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_INDEX,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Index)
    })
  ,

  'StyleTag' : _reflection.GeneratedProtocolMessageType('StyleTag', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_STYLETAG,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.StyleTag)
    })
  ,

  'ElementSelectorWithOptions' : _reflection.GeneratedProtocolMessageType('ElementSelectorWithOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTORWITHOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelectorWithOptions)
    })
  ,

  'ElementSelectorWithDuration' : _reflection.GeneratedProtocolMessageType('ElementSelectorWithDuration', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTORWITHDURATION,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelectorWithDuration)
    })
  ,

  'SelectElementSelector' : _reflection.GeneratedProtocolMessageType('SelectElementSelector', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_SELECTELEMENTSELECTOR,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.SelectElementSelector)
    })
  ,

  'JavascriptCode' : _reflection.GeneratedProtocolMessageType('JavascriptCode', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_JAVASCRIPTCODE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.JavascriptCode)
    })
  ,

  'RawJson' : _reflection.GeneratedProtocolMessageType('RawJson', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_RAWJSON,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.RawJson)
    })
  ,
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Request)
  })
_sym_db.RegisterMessage(Request)
_sym_db.RegisterMessage(Request.Empty)
_sym_db.RegisterMessage(Request.ScreenshotPath)
_sym_db.RegisterMessage(Request.Browser)
_sym_db.RegisterMessage(Request.Context)
_sym_db.RegisterMessage(Request.Url)
_sym_db.RegisterMessage(Request.TextInput)
_sym_db.RegisterMessage(Request.ElementProperty)
_sym_db.RegisterMessage(Request.TypeText)
_sym_db.RegisterMessage(Request.FillText)
_sym_db.RegisterMessage(Request.ClearText)
_sym_db.RegisterMessage(Request.PressKeys)
_sym_db.RegisterMessage(Request.ElementSelector)
_sym_db.RegisterMessage(Request.Timeout)
_sym_db.RegisterMessage(Request.Index)
_sym_db.RegisterMessage(Request.StyleTag)
_sym_db.RegisterMessage(Request.ElementSelectorWithOptions)
_sym_db.RegisterMessage(Request.ElementSelectorWithDuration)
_sym_db.RegisterMessage(Request.SelectElementSelector)
_sym_db.RegisterMessage(Request.JavascriptCode)
_sym_db.RegisterMessage(Request.RawJson)

SelectEntry = _reflection.GeneratedProtocolMessageType('SelectEntry', (_message.Message,), {
  'DESCRIPTOR' : _SELECTENTRY,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:SelectEntry)
  })
_sym_db.RegisterMessage(SelectEntry)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {

  'Empty' : _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_EMPTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Empty)
    })
  ,

  'String' : _reflection.GeneratedProtocolMessageType('String', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_STRING,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.String)
    })
  ,

  'Bool' : _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_BOOL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Bool)
    })
  ,

  'Int' : _reflection.GeneratedProtocolMessageType('Int', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_INT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Int)
    })
  ,

  'Select' : _reflection.GeneratedProtocolMessageType('Select', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_SELECT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Select)
    })
  ,

  'JavascriptExecutionResult' : _reflection.GeneratedProtocolMessageType('JavascriptExecutionResult', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_JAVASCRIPTEXECUTIONRESULT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.JavascriptExecutionResult)
    })
  ,
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Response)
  })
_sym_db.RegisterMessage(Response)
_sym_db.RegisterMessage(Response.Empty)
_sym_db.RegisterMessage(Response.String)
_sym_db.RegisterMessage(Response.Bool)
_sym_db.RegisterMessage(Response.Int)
_sym_db.RegisterMessage(Response.Select)
_sym_db.RegisterMessage(Response.JavascriptExecutionResult)



_PLAYWRIGHT = _descriptor.ServiceDescriptor(
  name='Playwright',
  full_name='Playwright',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1175,
  serialized_end=3370,
  methods=[
  _descriptor.MethodDescriptor(
    name='TakeScreenshot',
    full_name='Playwright.TakeScreenshot',
    index=0,
    containing_service=None,
    input_type=_REQUEST_SCREENSHOTPATH,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CloseBrowser',
    full_name='Playwright.CloseBrowser',
    index=1,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GoTo',
    full_name='Playwright.GoTo',
    index=2,
    containing_service=None,
    input_type=_REQUEST_URL,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GoBack',
    full_name='Playwright.GoBack',
    index=3,
    containing_service=None,
    input_type=_REQUEST_URL,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GoForward',
    full_name='Playwright.GoForward',
    index=4,
    containing_service=None,
    input_type=_REQUEST_URL,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetTitle',
    full_name='Playwright.GetTitle',
    index=5,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetElementCount',
    full_name='Playwright.GetElementCount',
    index=6,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_INT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='InputText',
    full_name='Playwright.InputText',
    index=7,
    containing_service=None,
    input_type=_REQUEST_TEXTINPUT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TypeText',
    full_name='Playwright.TypeText',
    index=8,
    containing_service=None,
    input_type=_REQUEST_TYPETEXT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='FillText',
    full_name='Playwright.FillText',
    index=9,
    containing_service=None,
    input_type=_REQUEST_FILLTEXT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClearText',
    full_name='Playwright.ClearText',
    index=10,
    containing_service=None,
    input_type=_REQUEST_CLEARTEXT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDomProperty',
    full_name='Playwright.GetDomProperty',
    index=11,
    containing_service=None,
    input_type=_REQUEST_ELEMENTPROPERTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBoolProperty',
    full_name='Playwright.GetBoolProperty',
    index=12,
    containing_service=None,
    input_type=_REQUEST_ELEMENTPROPERTY,
    output_type=_RESPONSE_BOOL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetTextContent',
    full_name='Playwright.GetTextContent',
    index=13,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUrl',
    full_name='Playwright.GetUrl',
    index=14,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Click',
    full_name='Playwright.Click',
    index=15,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Press',
    full_name='Playwright.Press',
    index=16,
    containing_service=None,
    input_type=_REQUEST_PRESSKEYS,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSelectContent',
    full_name='Playwright.GetSelectContent',
    index=17,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_SELECT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SelectOption',
    full_name='Playwright.SelectOption',
    index=18,
    containing_service=None,
    input_type=_REQUEST_SELECTELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeselectOption',
    full_name='Playwright.DeselectOption',
    index=19,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CheckCheckbox',
    full_name='Playwright.CheckCheckbox',
    index=20,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UncheckCheckbox',
    full_name='Playwright.UncheckCheckbox',
    index=21,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Health',
    full_name='Playwright.Health',
    index=22,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_STRING,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetTimeout',
    full_name='Playwright.SetTimeout',
    index=23,
    containing_service=None,
    input_type=_REQUEST_TIMEOUT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AddStyleTag',
    full_name='Playwright.AddStyleTag',
    index=24,
    containing_service=None,
    input_type=_REQUEST_STYLETAG,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HighlightElements',
    full_name='Playwright.HighlightElements',
    index=25,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTORWITHDURATION,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClickWithOptions',
    full_name='Playwright.ClickWithOptions',
    index=26,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTORWITHOPTIONS,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Focus',
    full_name='Playwright.Focus',
    index=27,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTOR,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WaitForElementsState',
    full_name='Playwright.WaitForElementsState',
    index=28,
    containing_service=None,
    input_type=_REQUEST_ELEMENTSELECTORWITHOPTIONS,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ExecuteJavascriptOnPage',
    full_name='Playwright.ExecuteJavascriptOnPage',
    index=29,
    containing_service=None,
    input_type=_REQUEST_JAVASCRIPTCODE,
    output_type=_RESPONSE_JAVASCRIPTEXECUTIONRESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetPageState',
    full_name='Playwright.GetPageState',
    index=30,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_JAVASCRIPTEXECUTIONRESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AutoActivatePages',
    full_name='Playwright.AutoActivatePages',
    index=31,
    containing_service=None,
    input_type=_REQUEST_EMPTY,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SwitchPage',
    full_name='Playwright.SwitchPage',
    index=32,
    containing_service=None,
    input_type=_REQUEST_INDEX,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SwitchContext',
    full_name='Playwright.SwitchContext',
    index=33,
    containing_service=None,
    input_type=_REQUEST_INDEX,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SwitchBrowser',
    full_name='Playwright.SwitchBrowser',
    index=34,
    containing_service=None,
    input_type=_REQUEST_INDEX,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreatePage',
    full_name='Playwright.CreatePage',
    index=35,
    containing_service=None,
    input_type=_REQUEST_URL,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateContext',
    full_name='Playwright.CreateContext',
    index=36,
    containing_service=None,
    input_type=_REQUEST_CONTEXT,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateBrowser',
    full_name='Playwright.CreateBrowser',
    index=37,
    containing_service=None,
    input_type=_REQUEST_BROWSER,
    output_type=_RESPONSE_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PLAYWRIGHT)

DESCRIPTOR.services_by_name['Playwright'] = _PLAYWRIGHT

# @@protoc_insertion_point(module_scope)



import json
from enum import Enum, auto

from robotlibcore import keyword  # type: ignore
from typing import Optional, Dict

from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils.time_conversion import timestr_to_millisecs


class ElementState(Enum):
    attached = auto()
    detached = auto()
    visible = auto()
    hidden = auto()


class Waiter(LibraryComponent):
    @keyword
    def wait_for_elements_state(
        self,
        selector: str,
        state: ElementState = ElementState.visible,
        timeout: Optional[str] = None,
    ):
        """Waits for the element found by ``selector`` to satisfy state option.

        State options could be either appear/disappear from dom, or become visible/hidden.
        If at the moment of calling the keyword, the selector already satisfies the condition,
        the keyword will return immediately.

        If the selector doesn't satisfy the condition within the timeout the keyword will FAIL.

        ``state``
        - ``attached``: to be present in DOM.
        - ``detached``: to not be present in DOM.
        - ``visible``: to have non-empty bounding box and no visibility:hidden.
        - ``hidden``: to be detached from DOM, or have an empty bounding box or visibility:hidden.

        Note that element without any content or with display:none has an empty bounding box
        and is not considered visible.

        ``timeout``: (optional) uses default timout if not set.
        """
        with self.playwright.grpc_channel() as stub:
            options: Dict[str, object] = {"state": state.name}
            if timeout:
                timeout_ms = timestr_to_millisecs(timeout)
                options["timeout"] = timeout_ms
            options_json = json.dumps(options)
            response = stub.WaitForElementsState(
                Request().ElementSelectorWithOptions(
                    selector=selector, options=options_json
                )
            )
            self.info(response.log)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from Browser.generated import playwright_pb2 as playwright__pb2


class PlaywrightStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.TakeScreenshot = channel.unary_unary(
                '/Playwright/TakeScreenshot',
                request_serializer=playwright__pb2.Request.ScreenshotPath.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.CloseBrowser = channel.unary_unary(
                '/Playwright/CloseBrowser',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoTo = channel.unary_unary(
                '/Playwright/GoTo',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoBack = channel.unary_unary(
                '/Playwright/GoBack',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GoForward = channel.unary_unary(
                '/Playwright/GoForward',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetTitle = channel.unary_unary(
                '/Playwright/GetTitle',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetElementCount = channel.unary_unary(
                '/Playwright/GetElementCount',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Int.FromString,
                )
        self.InputText = channel.unary_unary(
                '/Playwright/InputText',
                request_serializer=playwright__pb2.Request.TextInput.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.TypeText = channel.unary_unary(
                '/Playwright/TypeText',
                request_serializer=playwright__pb2.Request.TypeText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.FillText = channel.unary_unary(
                '/Playwright/FillText',
                request_serializer=playwright__pb2.Request.FillText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.ClearText = channel.unary_unary(
                '/Playwright/ClearText',
                request_serializer=playwright__pb2.Request.ClearText.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetDomProperty = channel.unary_unary(
                '/Playwright/GetDomProperty',
                request_serializer=playwright__pb2.Request.ElementProperty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetBoolProperty = channel.unary_unary(
                '/Playwright/GetBoolProperty',
                request_serializer=playwright__pb2.Request.ElementProperty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Bool.FromString,
                )
        self.GetTextContent = channel.unary_unary(
                '/Playwright/GetTextContent',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.GetUrl = channel.unary_unary(
                '/Playwright/GetUrl',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.Click = channel.unary_unary(
                '/Playwright/Click',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Press = channel.unary_unary(
                '/Playwright/Press',
                request_serializer=playwright__pb2.Request.PressKeys.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.GetSelectContent = channel.unary_unary(
                '/Playwright/GetSelectContent',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Select.FromString,
                )
        self.SelectOption = channel.unary_unary(
                '/Playwright/SelectOption',
                request_serializer=playwright__pb2.Request.SelectElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.DeselectOption = channel.unary_unary(
                '/Playwright/DeselectOption',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CheckCheckbox = channel.unary_unary(
                '/Playwright/CheckCheckbox',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.UncheckCheckbox = channel.unary_unary(
                '/Playwright/UncheckCheckbox',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Health = channel.unary_unary(
                '/Playwright/Health',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.String.FromString,
                )
        self.SetTimeout = channel.unary_unary(
                '/Playwright/SetTimeout',
                request_serializer=playwright__pb2.Request.Timeout.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.AddStyleTag = channel.unary_unary(
                '/Playwright/AddStyleTag',
                request_serializer=playwright__pb2.Request.StyleTag.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.HighlightElements = channel.unary_unary(
                '/Playwright/HighlightElements',
                request_serializer=playwright__pb2.Request.ElementSelectorWithDuration.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.ClickWithOptions = channel.unary_unary(
                '/Playwright/ClickWithOptions',
                request_serializer=playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.Focus = channel.unary_unary(
                '/Playwright/Focus',
                request_serializer=playwright__pb2.Request.ElementSelector.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.WaitForElementsState = channel.unary_unary(
                '/Playwright/WaitForElementsState',
                request_serializer=playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.ExecuteJavascriptOnPage = channel.unary_unary(
                '/Playwright/ExecuteJavascriptOnPage',
                request_serializer=playwright__pb2.Request.JavascriptCode.SerializeToString,
                response_deserializer=playwright__pb2.Response.JavascriptExecutionResult.FromString,
                )
        self.GetPageState = channel.unary_unary(
                '/Playwright/GetPageState',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.JavascriptExecutionResult.FromString,
                )
        self.AutoActivatePages = channel.unary_unary(
                '/Playwright/AutoActivatePages',
                request_serializer=playwright__pb2.Request.Empty.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SwitchPage = channel.unary_unary(
                '/Playwright/SwitchPage',
                request_serializer=playwright__pb2.Request.Index.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SwitchContext = channel.unary_unary(
                '/Playwright/SwitchContext',
                request_serializer=playwright__pb2.Request.Index.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.SwitchBrowser = channel.unary_unary(
                '/Playwright/SwitchBrowser',
                request_serializer=playwright__pb2.Request.Index.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CreatePage = channel.unary_unary(
                '/Playwright/CreatePage',
                request_serializer=playwright__pb2.Request.Url.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CreateContext = channel.unary_unary(
                '/Playwright/CreateContext',
                request_serializer=playwright__pb2.Request.Context.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )
        self.CreateBrowser = channel.unary_unary(
                '/Playwright/CreateBrowser',
                request_serializer=playwright__pb2.Request.Browser.SerializeToString,
                response_deserializer=playwright__pb2.Response.Empty.FromString,
                )


class PlaywrightServicer(object):
    """Missing associated documentation comment in .proto file."""

    def TakeScreenshot(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoTo(self, request, context):
        """Opens the url in currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoBack(self, request, context):
        """Navigate to the next page in history 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GoForward(self, request, context):
        """Navigate to the previous page in history. 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTitle(self, request, context):
        """Gets title of currently open Playwright page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetElementCount(self, request, context):
        """Returns the count of elements found with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InputText(self, request, context):
        """Wraps playwrights page.fill to input text into input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TypeText(self, request, context):
        """Wraps playwrights page.type to type text into input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FillText(self, request, context):
        """Wraps playwrights page.fill to fill text of input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClearText(self, request, context):
        """Wraps playwrights page.fill with empty text to clear input specified with selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDomProperty(self, request, context):
        """Gets the DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBoolProperty(self, request, context):
        """Gets the boolean DOM property 'property' of selector specified element 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTextContent(self, request, context):
        """Wraps playwrights page.textContent, returns textcontent of element by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUrl(self, request, context):
        """*Returns current playwright page url
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Click(self, request, context):
        """Clicks element specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Press(self, request, context):
        """Inputs a list of keypresses to element specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSelectContent(self, request, context):
        """Gets the Select element specified by selector and returns the contents 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SelectOption(self, request, context):
        """Selects option matching matcher in Select element matching selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeselectOption(self, request, context):
        """Deselects options in Select element matching selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckCheckbox(self, request, context):
        """Checks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UncheckCheckbox(self, request, context):
        """Unchecks checkbox specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Health(self, request, context):
        """Health check endpoint for the service 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTimeout(self, request, context):
        """Set's  playwright timeout 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddStyleTag(self, request, context):
        """Adds a <style> to head of site. 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HighlightElements(self, request, context):
        """Highlights elements matching selector for duration 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClickWithOptions(self, request, context):
        """Clicks element specified by selector and options 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Focus(self, request, context):
        """Focuses element specified by selector 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitForElementsState(self, request, context):
        """Waits for element be in a specific state
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteJavascriptOnPage(self, request, context):
        """Executes javascript on the active page 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPageState(self, request, context):
        """Get Page State JSON 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AutoActivatePages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchPage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SwitchBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateBrowser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PlaywrightServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'TakeScreenshot': grpc.unary_unary_rpc_method_handler(
                    servicer.TakeScreenshot,
                    request_deserializer=playwright__pb2.Request.ScreenshotPath.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'CloseBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseBrowser,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoTo': grpc.unary_unary_rpc_method_handler(
                    servicer.GoTo,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoBack': grpc.unary_unary_rpc_method_handler(
                    servicer.GoBack,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GoForward': grpc.unary_unary_rpc_method_handler(
                    servicer.GoForward,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetTitle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTitle,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetElementCount': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElementCount,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Int.SerializeToString,
            ),
            'InputText': grpc.unary_unary_rpc_method_handler(
                    servicer.InputText,
                    request_deserializer=playwright__pb2.Request.TextInput.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'TypeText': grpc.unary_unary_rpc_method_handler(
                    servicer.TypeText,
                    request_deserializer=playwright__pb2.Request.TypeText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'FillText': grpc.unary_unary_rpc_method_handler(
                    servicer.FillText,
                    request_deserializer=playwright__pb2.Request.FillText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'ClearText': grpc.unary_unary_rpc_method_handler(
                    servicer.ClearText,
                    request_deserializer=playwright__pb2.Request.ClearText.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetDomProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDomProperty,
                    request_deserializer=playwright__pb2.Request.ElementProperty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetBoolProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBoolProperty,
                    request_deserializer=playwright__pb2.Request.ElementProperty.FromString,
                    response_serializer=playwright__pb2.Response.Bool.SerializeToString,
            ),
            'GetTextContent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTextContent,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'GetUrl': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUrl,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'Click': grpc.unary_unary_rpc_method_handler(
                    servicer.Click,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Press': grpc.unary_unary_rpc_method_handler(
                    servicer.Press,
                    request_deserializer=playwright__pb2.Request.PressKeys.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'GetSelectContent': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSelectContent,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Select.SerializeToString,
            ),
            'SelectOption': grpc.unary_unary_rpc_method_handler(
                    servicer.SelectOption,
                    request_deserializer=playwright__pb2.Request.SelectElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'DeselectOption': grpc.unary_unary_rpc_method_handler(
                    servicer.DeselectOption,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckCheckbox,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'UncheckCheckbox': grpc.unary_unary_rpc_method_handler(
                    servicer.UncheckCheckbox,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Health': grpc.unary_unary_rpc_method_handler(
                    servicer.Health,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.String.SerializeToString,
            ),
            'SetTimeout': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTimeout,
                    request_deserializer=playwright__pb2.Request.Timeout.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'AddStyleTag': grpc.unary_unary_rpc_method_handler(
                    servicer.AddStyleTag,
                    request_deserializer=playwright__pb2.Request.StyleTag.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'HighlightElements': grpc.unary_unary_rpc_method_handler(
                    servicer.HighlightElements,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithDuration.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'ClickWithOptions': grpc.unary_unary_rpc_method_handler(
                    servicer.ClickWithOptions,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'Focus': grpc.unary_unary_rpc_method_handler(
                    servicer.Focus,
                    request_deserializer=playwright__pb2.Request.ElementSelector.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'WaitForElementsState': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitForElementsState,
                    request_deserializer=playwright__pb2.Request.ElementSelectorWithOptions.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'ExecuteJavascriptOnPage': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteJavascriptOnPage,
                    request_deserializer=playwright__pb2.Request.JavascriptCode.FromString,
                    response_serializer=playwright__pb2.Response.JavascriptExecutionResult.SerializeToString,
            ),
            'GetPageState': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPageState,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.JavascriptExecutionResult.SerializeToString,
            ),
            'AutoActivatePages': grpc.unary_unary_rpc_method_handler(
                    servicer.AutoActivatePages,
                    request_deserializer=playwright__pb2.Request.Empty.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SwitchPage': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchPage,
                    request_deserializer=playwright__pb2.Request.Index.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SwitchContext': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchContext,
                    request_deserializer=playwright__pb2.Request.Index.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'SwitchBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.SwitchBrowser,
                    request_deserializer=playwright__pb2.Request.Index.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CreatePage': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePage,
                    request_deserializer=playwright__pb2.Request.Url.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CreateContext': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateContext,
                    request_deserializer=playwright__pb2.Request.Context.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
            'CreateBrowser': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateBrowser,
                    request_deserializer=playwright__pb2.Request.Browser.FromString,
                    response_serializer=playwright__pb2.Response.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Playwright', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Playwright(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def TakeScreenshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/TakeScreenshot',
            playwright__pb2.Request.ScreenshotPath.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CloseBrowser',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoTo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoTo',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoBack(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoBack',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GoForward(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GoForward',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTitle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetTitle',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetElementCount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetElementCount',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Int.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def InputText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/InputText',
            playwright__pb2.Request.TextInput.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TypeText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/TypeText',
            playwright__pb2.Request.TypeText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FillText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/FillText',
            playwright__pb2.Request.FillText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClearText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ClearText',
            playwright__pb2.Request.ClearText.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDomProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetDomProperty',
            playwright__pb2.Request.ElementProperty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBoolProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetBoolProperty',
            playwright__pb2.Request.ElementProperty.SerializeToString,
            playwright__pb2.Response.Bool.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTextContent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetTextContent',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUrl(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetUrl',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Click(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Click',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Press(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Press',
            playwright__pb2.Request.PressKeys.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSelectContent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetSelectContent',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Select.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SelectOption(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SelectOption',
            playwright__pb2.Request.SelectElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeselectOption(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/DeselectOption',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CheckCheckbox',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UncheckCheckbox(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/UncheckCheckbox',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Health(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Health',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.String.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTimeout(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SetTimeout',
            playwright__pb2.Request.Timeout.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddStyleTag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/AddStyleTag',
            playwright__pb2.Request.StyleTag.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def HighlightElements(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/HighlightElements',
            playwright__pb2.Request.ElementSelectorWithDuration.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClickWithOptions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ClickWithOptions',
            playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Focus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/Focus',
            playwright__pb2.Request.ElementSelector.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitForElementsState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/WaitForElementsState',
            playwright__pb2.Request.ElementSelectorWithOptions.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecuteJavascriptOnPage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/ExecuteJavascriptOnPage',
            playwright__pb2.Request.JavascriptCode.SerializeToString,
            playwright__pb2.Response.JavascriptExecutionResult.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPageState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/GetPageState',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.JavascriptExecutionResult.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AutoActivatePages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/AutoActivatePages',
            playwright__pb2.Request.Empty.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchPage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchPage',
            playwright__pb2.Request.Index.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchContext',
            playwright__pb2.Request.Index.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SwitchBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/SwitchBrowser',
            playwright__pb2.Request.Index.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreatePage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CreatePage',
            playwright__pb2.Request.Url.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CreateContext',
            playwright__pb2.Request.Context.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateBrowser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Playwright/CreateBrowser',
            playwright__pb2.Request.Browser.SerializeToString,
            playwright__pb2.Response.Empty.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import Optional, Union

from robotlibcore import keyword  # type: ignore

from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request
from ..utils.time_conversion import timestr_to_millisecs


class Control(LibraryComponent):
    """Keywords to do things on the current browser page and modify the page
    """

    def __init__(self, library):
        self.library = library

    @property
    def playwright(self):
        return self.library.playwright

    @keyword
    def go_forward(self):
        """Navigates to the next page in history."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoForward(Request.Empty())
            self.info(response.log)

    @keyword
    def go_back(self):
        """Navigates to the previous page in history."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoBack(Request.Empty())
            self.info(response.log)

    @keyword
    def go_to(self, url: str):
        """Navigates to the given ``url``."""
        with self.playwright.grpc_channel() as stub:
            response = stub.GoTo(Request().Url(url=url))
            self.info(response.log)

    @keyword
    def take_page_screenshot(self, path: Optional[str] = None):
        """Takes screenshot of the current window and saves it to ``path``.

        The default path is the Robot Framework output directory.
        """
        if path is None:
            path = self.library.get_screenshot_path
        self.info(f"Taking screenshot into ${path}")
        with self.playwright.grpc_channel() as stub:
            response = stub.TakeScreenshot(Request().ScreenshotPath(path=path))
            self.info(
                f"Saved screenshot in <a href='file://{response.body}''>{response.body}</a>",
                html=True,
            )

    @keyword
    def set_timeout(self, timeout: str):
        """Sets the timeout used by most input and getter keywords.

        Technically this is the timeout of current playwright context.
        """
        parsed_timeout = timestr_to_millisecs(timeout)
        with self.playwright.grpc_channel() as stub:
            response = stub.SetTimeout(Request().Timeout(timeout=parsed_timeout))
            self.info(response.log)

    @keyword
    def add_style_tag(self, content: str):
        """Adds a <style type="text/css"> tag with the content.

        ``content``: Raw CSS content to be injected into frame.
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.AddStyleTag(Request().StyleTag(content=content))
            self.info(response.log)

    @keyword
    def highlight_element(self, selector: str, duration: Union[str, int] = "5s"):
        """Adds a red highlight to elements matched by ``selector`` for ``duration``"""
        with self.playwright.grpc_channel() as stub:
            duration_ms = timestr_to_millisecs(duration)
            response = stub.HighlightElements(
                Request().ElementSelectorWithDuration(
                    selector=selector, duration=duration_ms
                )
            )
            self.info(response.log)

    @keyword
    def switch_active_page(self, index: int):
        """Switches the active browser page to another open page by ``index``.

            Newly opened pages get appended to the end of the list
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.SwitchActivePage(Request().Index(index=index))
            self.info(response.log)

    @keyword
    def auto_activate_pages(self):
        """Toggles automatically changing active page to latest opened page """
        with self.playwright.grpc_channel() as stub:
            response = stub.AutoActivatePages(Request().Empty())
            self.info(response.log)



from robot.api import logger  # type: ignore


class LibraryComponent:
    def __init__(self, library):
        """Base class exposing attributes from the common context.

        :param library: The library itself as a context object.
        :type library: Browser.Browser
        """
        self.library = library

    def info(self, msg: str, html=False):
        logger.info(msg, html)

    def debug(self, msg: str, html=False):
        logger.debug(msg, html)

    @property
    def playwright(self):
        return self.library.playwright



import json
from typing import Optional, Any

from robotlibcore import keyword  # type: ignore

from ..assertion_engine import AssertionOperator, verify_assertion
from ..base import LibraryComponent
from ..generated.playwright_pb2 import Request


class WebAppState(LibraryComponent):
    @keyword
    def local_storage_get(
        self,
        key: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """
        Get saved data from from localStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(script=f'window.localStorage.getItem("{key}")')
            )
            self.info(response.log)
        return verify_assertion(
            json.loads(response.result),
            assertion_operator,
            assertion_expected,
            "localStorage ",
        )

    @keyword
    def local_storage_set(self, key: str, value: str):
        """
        Save data to localStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(
                    script=f'window.localStorage.setItem("{key}", "{value}")'
                )
            )
            self.info(response.log)

    @keyword
    def local_storage_remove(self, key: str):
        """
        Remove saved data with key from localStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(
                    script=f'window.localStorage.removeItem("{key}")'
                )
            )
            self.info(response.log)

    @keyword
    def local_storage_clear(self):
        """
        Remove all saved data from localStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(script="window.localStorage.clear()")
            )
            self.info(response.log)

    @keyword
    def session_storage_get(
        self,
        key: str,
        assertion_operator: Optional[AssertionOperator] = None,
        assertion_expected: Any = None,
    ) -> Any:
        """
        Get saved data from from sessionStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.getItem("{key}")'
                )
            )
            self.info(response.log)
        return verify_assertion(
            json.loads(response.result),
            assertion_operator,
            assertion_expected,
            "sessionStorage ",
        )

    @keyword
    def session_storage_set(self, key: str, value: str):
        """
        Save data to sessionStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.setItem("{key}", "{value}")'
                )
            )
            self.info(response.log)

    @keyword
    def session_storage_remove(self, key: str):
        """
        Remove saved data with key from sessionStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(
                    script=f'window.sessionStorage.removeItem("{key}")'
                )
            )
            self.info(response.log)

    @keyword
    def session_storage_clear(self):
        """
        Remove all saved data from sessionStorage
        """
        with self.playwright.grpc_channel() as stub:
            response = stub.ExecuteJavascriptOnPage(
                Request().JavascriptCode(script="window.sessionStorage.clear()")
            )
            self.info(response.log)
